{{
sidebar_item(
'Home', 
icon='fas fa-user-lock', 
url=url_for('appadmin.user_list')
)
}}


{{
sidebar_item(
'Add', 
icon='fas fa-plus-circle', 
url=url_for('appadmin.user_add')
)
}}


{{
sidebar_item(
'Roles', 
icon='fa fa-shield-alt', 
url=url_for('appadmin.roles')
)
}}